package main;
import java.io.File;
import java.io.IOException;
import java.net.Socket;
import java.util.ArrayList;

import manager.IManager;
import manager.MenuManager;
import manager.MilitaryManager;
import manager.ResourceExplorationManager;
import manager.StandardManager;
import model.UnitPrototype;
import model.World;
import msgs.ByteConvert;
import msgs.Message;

//--------------------------------------------------------------------
public class BotMain 
{

	//--------------------------------------------------------------------
	public enum BotStates
	{
		NOT_CONNECTED, CONNECTED, QUIT 
	};
	private BotStates botState;

	public String botName;
	public int playerNr = -1;
	public int nrPlayers = 0;
	public int credits = 0;
	public static final int CLANS_ON = 0;
	public int clans = 1; // 1: clans off
	public boolean isLoadedGame = false;

	public World world = new World ();
	
	private ArrayList<IManager> managers = null;

	//--------------------------------------------------------------------
	public void setClientState (BotStates newState) 
	{
		botState = newState;
	}

	//--------------------------------------------------------------------
	public static BotMain getInstance () 
	{
		if (instance == null) 
			instance = new BotMain ();
		return instance;
	}	
	private static BotMain instance = null;
	
	//--------------------------------------------------------------------
	public static Connection getConnection ()
	{
		return getInstance ().connection;
	}
	
	//--------------------------------------------------------------------
	private BotMain () 
	{
		setClientState (BotStates.NOT_CONNECTED);
		managers = new ArrayList<IManager> ();
		managers.add (new MenuManager ());
		managers.add (new StandardManager ());		
		managers.add (new MilitaryManager ());
		managers.add (new ResourceExplorationManager ());
	}

	//--------------------------------------------------------------------
	private void run (String ip, int port, String _botName) 
	{
		botName = _botName;
		boolean running = true;
		while (running) 
		{
			if (botState == BotStates.NOT_CONNECTED) 
			{
				connectToServer (ip, port);
				if (connection != null && connection.isConnected ())
					botState = BotStates.CONNECTED;
			}
			else if (botState == BotStates.CONNECTED)
			{
				runGame ();
			}
			else if (botState == BotStates.QUIT) 
			{
				System.out.println ("Quit");
				running = false;
				try { Thread.sleep (500); } catch (InterruptedException e) {}
			}
			else
			{
				System.out.println ("ERROR: ILLEGAL STATE");
				running = false;
			}
		}
		if (connection != null && connection.isConnected ())
			connection.close ();
	}
	
	//--------------------------------------------------------------------
	private void connectToServer (String ip, int port)
	{
		try 
		{
			Socket socket = new Socket (ip, port);
			connection = new Connection (socket);
		} 
		catch (IOException e) 
		{
			System.out.println ("Can't connect to server " + ip + " on Port " + port +": " + e);
			botState = BotStates.QUIT;
		}
	}
	private Connection connection = null;
	
	//--------------------------------------------------------------------
	private void runGame ()
	{
		while (connection.isConnected ())
		{
			while (connection.msgAvailable ())
			{
				Message msg = connection.getNextMsg ();
				for (int mgrIdx = 0; mgrIdx < managers.size (); mgrIdx++)
				{
					managers.get (mgrIdx).receiveMessage (msg);
					msg.reinitMsgStream ();
				}
			}
			for (int mgrIdx = 0; mgrIdx < managers.size (); mgrIdx++)
				managers.get (mgrIdx).act ();
			
			try { Thread.sleep (50); } catch (InterruptedException e) {}
		}
	}

//	//--------------------------------------------------------------------
//	private void sendRequestResync ()
//	{
//		BasicMsg requestResyncMsg = new BasicMsg (Message.GAME_EV_REQUEST_RESYNC, playerNr);
//		requestResyncMsg.pushByte (playerNr);
//		connection.sendMsg (requestResyncMsg);
//		System.out.println ("Sent GAME_EV_REQUEST_RESYNC");		
//	}
	
	
	//--------------------------------------------------------------------
	// MAIN
	//--------------------------------------------------------------------
	public static void main (String[] args) 
	{
		byte[] testData = new byte[16];
		ByteConvert.fromInt16 (203, testData, 0);
		System.out.println ("Result 203: " + ByteConvert.toInt16 (testData, 0));
		ByteConvert.fromInt16 (15, testData, 0);
		System.out.println ("Result 15: " + ByteConvert.toInt16 (testData, 0));
		ByteConvert.fromInt16 (1026, testData, 0);
		System.out.println ("Result 1026: " + ByteConvert.toInt16 (testData, 0));
		ByteConvert.fromInt16 (-1, testData, 0);
		System.out.println ("Result -1:" + ByteConvert.toInt16 (testData, 0));
		ByteConvert.fromInt16 (-12431, testData, 0);
		System.out.println ("Result -12431: " + ByteConvert.toInt16 (testData, 0));
		ByteConvert.fromInt32 (15, testData, 0);
		System.out.println ("Result 15: " + ByteConvert.toInt32 (testData, 0));
		ByteConvert.fromInt32 (2041, testData, 0);
		System.out.println ("Result 2041: " + ByteConvert.toInt32 (testData, 0));
		
		File testXMLFile = new File (new File ("").getAbsoluteFile ().getParentFile (), "buildings" + File.separatorChar + "fac_air" + File.separatorChar + "data.xml");
		UnitPrototype testProto = new UnitPrototype (testXMLFile.toString ());
		System.out.println (testProto.toString ());
		File testXMLFileDepot = new File (new File ("").getAbsoluteFile ().getParentFile (), "buildings" + File.separatorChar + "depot" + File.separatorChar + "data.xml");
		UnitPrototype testProtoDepot = new UnitPrototype (testXMLFileDepot.toString ());
		System.out.println (testProtoDepot.toString ());
		
		String ip = "127.0.0.1";
		int port = 58600;
		String botname = "StupidAnxious";
		
		if (args.length < 3)
		{
			System.out.println ("Usage: java BotMain [xxx.xxx.xxx.xxx xxxxx Botname]");
			System.out.println ("Now using default settings...");
		}
		else
		{
			ip = args[0];
			port = new Integer (args[1]);
			botname = args[2];
		}
		System.out.println ("Connecting to " + ip + ":" + port + " as " + botname);
		BotMain.getInstance ().run (ip, port, botname);
	}

}
