package model;

import java.io.File;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

//--------------------------------------------------------------------
public class UnitPrototype 
{

	UnitType type = new UnitType (-1, -1);
	String name = "";

	int ammoMax = 0;
	int shotsMax = 0;
	int range = 0;
	int damage = 0;
	int armor = 0;
	int hitpointsMax = 0;
	int scan = 0;

	public static final int TERRAIN_NONE = 0;
	public static final int TERRAIN_AIR = 1;
	public static final int TERRAIN_SEA = 2;
	public static final int TERRAIN_GROUND = 4;
	public static final int TERRAIN_COAST = 8;
	public static final int AREA_SUB = 16;
	public static final int AREA_EXP_MINE = 32;
	byte canAttack = TERRAIN_NONE;
	boolean canDriveAndFire = false;

	int buildCosts = 0;
	String canBuild = ""; // e.g. "Plane", "BigBuilding"
	String buildAs = ""; // e.g. "BigGroundVehicle", "BigBuilding"
	//boolean buildIntern = false; 
	int maxBuildFactor = 1;

	int speedMax = 0;
	float factorGround = 0;
	float factorSea = 0;
	float factorAir = 0;
	float factorCoast = 0;
	float modifiesSpeed = 0; // buildings like roads can modify the speed

	boolean isBig = false;
	boolean connectsToBase = false;
	boolean canClearArea = false;
	boolean canBeCaptured = false;
	boolean canBeDisabled = false;
	boolean canCapture = false;
	boolean canDisable = false;
	boolean canRepair = false;
	boolean canRearm = false;
	boolean canResearch = false;
	boolean canPlaceMines = false;
	boolean canSurvey = false;
	boolean doesSelfRepair = false;
	int convertsGold = 0;
	boolean canSelfDestroy = false;
	int canMineMaxRes = 0;

	int needsMetal = 0;
	int needsOil = 0;
	int needsEnergy = 0;
	int needsHumans = 0;
	int produceEnergy = 0;
	int produceHumans = 0;

	byte isStealthOn = TERRAIN_NONE;
	byte canDetectStealthOn = TERRAIN_NONE;

	public static final int SURFACE_POS_BENEATH_SEA = 0;
	public static final int SURFACE_POS_ABOVE_SEA = 1;
	public static final int SURFACE_POS_BASE = 2;
	public static final int SURFACE_POS_ABOVE_BASE = 3;
	public static final int SURFACE_POS_GROUND = 4;
	public static final int SURFACE_POS_ABOVE = 5;
	int surfacePosition = SURFACE_POS_GROUND;

	public static final int OVERBUILD_TYPE_NO = 0;
	public static final int	OVERBUILD_TYPE_YES = 1;
	public static final int OVERBUILD_TYPE_YESNREMOVE = 2;
	int canBeOverbuild = OVERBUILD_TYPE_NO;

	boolean canBeLandedOn = false;
	boolean canWork = false;
	boolean explodesOnContact = false;
	boolean isHuman = false;

	// Storage
	public static final int	STORE_RES_NONE = 0;
	public static final int	STORE_RES_METAL = 1;
	public static final int STORE_RES_OIL = 2;
	public static final int STORE_RES_GOLD = 3;
	int storeResType = STORE_RES_NONE;
	int storageResMax = 0;

	int storageUnitsMax = 0;
	ArrayList<String> storeUnitsTypes = new ArrayList<String> (); // type of the units, that can be stored inside
	String isStorageType = ""; // type of the unit, when it is stored somewhere

	//--------------------------------------------------------------------
	public UnitPrototype (String xmlFileName)
	{
		initFromXML (xmlFileName);
	}

	//--------------------------------------------------------------------
	private void initFromXML (String xmlFileName)
	{
		try 
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance ();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder ();
			Document doc = docBuilder.parse (new File (xmlFileName));

			canAttack = (byte)getIntValue (doc, "Can_Attack", "Num", canAttack);
			canDriveAndFire = getBooleanValue (doc, "Can_Drive_And_Fire", "YN", canDriveAndFire);
			
			buildCosts = getIntValue (doc, "Built_Costs", "Num", buildCosts);
			canBuild = getStringValue (doc, "Can_Build", "String", canBuild);
			buildAs = getStringValue (doc, "Build_As", "String", buildAs);
			maxBuildFactor = getIntValue (doc, "Max_Build_Factor", "Num", maxBuildFactor);
			
			factorGround = getFloatValue (doc, "Factor_Ground", "Num", factorGround);
			factorSea = getFloatValue (doc, "Factor_Sea", "Num", factorSea);
			factorAir = getFloatValue (doc, "Factor_Air", "Num", factorAir);
			factorCoast = getFloatValue (doc, "Factor_Coast", "Num", factorCoast);
			modifiesSpeed = getFloatValue (doc, "Modifies_Speed", "Num", modifiesSpeed);
			
			isBig = getBooleanValue (doc, "Is_Big", "YN", isBig);
			connectsToBase = getBooleanValue (doc, "Connects_To_Base", "YN", connectsToBase);
			canClearArea = getBooleanValue (doc, "Can_Clear_Area", "YN", canClearArea);
			canBeCaptured = getBooleanValue (doc, "Can_Be_Captured", "YN", canBeCaptured);
			canBeDisabled = getBooleanValue (doc, "Can_Be_Disabled", "YN", canBeDisabled);
			canCapture = getBooleanValue (doc, "Can_Capture", "YN", canCapture);
			canDisable = getBooleanValue (doc, "Can_Disable", "YN", canDisable);
			canRepair = getBooleanValue (doc, "Can_Repair", "YN", canRepair);
			canRearm = getBooleanValue (doc, "Can_Rearm", "YN", canRearm);
			canResearch = getBooleanValue (doc, "Can_Research", "YN", canResearch);
			canPlaceMines = getBooleanValue (doc, "Can_Place_Mines", "YN", canPlaceMines);
			canSurvey = getBooleanValue (doc, "Can_Survey", "YN", canSurvey);
			doesSelfRepair = getBooleanValue (doc, "Does_Self_Repair", "YN", doesSelfRepair);
			convertsGold = getIntValue (doc, "Converts_Gold", "Num", convertsGold);
			canSelfDestroy = getBooleanValue (doc, "Can_Self_Destroy", "YN", canSelfDestroy);
			canMineMaxRes = getIntValue (doc, "Can_Mine_Max_Resource", "Num", canMineMaxRes);
			
			//...
		}
		catch (SAXParseException err) 
		{
			System.out.println ("UnitPrototype.initFromXML (" + xmlFileName + "): Parsing error" + ", line " 
					+ err.getLineNumber () + ", uri " + err.getSystemId ());
			System.out.println(" " + err.getMessage ());
		}
		catch (SAXException e) 
		{
			Exception x = e.getException ();
			((x == null) ? e : x).printStackTrace ();
		} 
		catch (Throwable t) { t.printStackTrace (); }
	}
	
	//--------------------------------------------------------------------
	private int getIntValue (Document doc, String tagName, String attrName, int defaultValue)
	{
		String value = getStringValue (doc, tagName, attrName, "");
		if (value.equals (""))
			return defaultValue;
		int result = defaultValue;
		try { result = Integer.valueOf (value); }
		catch (NumberFormatException e) { System.out.println ("Could not parse integer value: " + value); }
		return result;
	}

	//--------------------------------------------------------------------
	private float getFloatValue (Document doc, String tagName, String attrName, float defaultValue)
	{
		String value = getStringValue (doc, tagName, attrName, "");
		if (value.equals (""))
			return defaultValue;
		float result = defaultValue;
		try { result = Float.valueOf (value); }
		catch (NumberFormatException e) { System.out.println ("Could not parse float value: " + value); }
		return result;
	}

	//--------------------------------------------------------------------
	private boolean getBooleanValue (Document doc, String tagName, String attrName, boolean defaultValue)
	{
		String value = getStringValue (doc, tagName, attrName, "");
		if (value.equals (""))
			return defaultValue;
		if (value.compareToIgnoreCase ("yes") == 0
			|| value.compareToIgnoreCase ("ja") == 0)
			return true;
		return false;
	}

	//--------------------------------------------------------------------
	private String getStringValue (Document doc, String tagName, String attrName, String defaultValue)
	{
		NodeList nodes = doc.getElementsByTagName (tagName);
		if (nodes.getLength () == 0)
			return defaultValue;
		Node node = nodes.item (0);
		if (node == null)
			return defaultValue;
		NamedNodeMap attributes = node.getAttributes ();
		if (attributes == null)
			return defaultValue;
		Node attribute = attributes.getNamedItem (attrName);
		if (attribute == null)
			return defaultValue;
		return attribute.getNodeValue ();
	}

}
