package msgs;

import java.io.IOException;
import java.io.OutputStream;

//--------------------------------------------------------------------
public class BasicMsg implements Message {

	//--------------------------------------------------------------------
	protected int msgType = MSG_UNKNOWN;
	protected int msgLength = 0;
	protected int curDataPos = 0;
	protected int playerNr = -1;
	byte[] data = null;
	
	//--------------------------------------------------------------------
	public BasicMsg (byte[] _data)
	{
		data = _data;
		initMsg ();
	}
	
	//--------------------------------------------------------------------
	public BasicMsg (int _msgType, int _playerNr)
	{
		data = new byte[1024];
		playerNr = _playerNr;
		msgType = _msgType;
		msgLength = 6;
		curDataPos = 6;
	}
	
	//--------------------------------------------------------------------
	public int getMsgType () 
	{
		return msgType;
	}
	
	//--------------------------------------------------------------------
	public int getPlayerNr ()
	{
		return playerNr;
	}
	
	//--------------------------------------------------------------------
	/** Needed if data from a msg is popped by different objects that don't know of each other. */
	public void reinitMsgStream ()
	{
		initMsg ();
	}

	//--------------------------------------------------------------------
	public void pushBoolean (boolean value)
	{
		ByteConvert.fromBoolean (value, data, curDataPos);
		msgLength += 1;
		curDataPos += 1;
	}

	//--------------------------------------------------------------------
	public void pushByte (int value)
	{
		ByteConvert.fromByte (value, data, curDataPos);
		msgLength += 1;
		curDataPos += 1;
	}

	//--------------------------------------------------------------------
	public void pushInt16 (int value)
	{
		ByteConvert.fromInt16 (value, data, curDataPos);
		msgLength += 2;
		curDataPos += 2;
	}
	
	//--------------------------------------------------------------------
	public void pushInt32 (int value)
	{
		ByteConvert.fromInt32 (value, data, curDataPos);
		msgLength += 4;
		curDataPos += 4;
	}
	
	//--------------------------------------------------------------------
	public void pushString (String value)
	{
		curDataPos = ByteConvert.fromString (value, data, curDataPos);
		msgLength = curDataPos;
	}
	
	
	//--------------------------------------------------------------------
	public int popInt32 ()
	{
		if (curDataPos - 3 < 6)
		{
			System.out.println ("Error: popInt32 before message start!");
			return 0;
		}
		curDataPos -= 3;
		int result = ByteConvert.toInt32 (data, curDataPos);
		curDataPos -= 1;
		return result;
	}

	//--------------------------------------------------------------------
	public int popInt16 ()
	{
		if (curDataPos - 1 < 6)
		{
			System.out.println ("Error: popInt16 before message start!");
			return 0;
		}
		curDataPos -= 1;
		int result = ByteConvert.toInt16 (data, curDataPos);
		curDataPos -= 1;
		return result;
	}

	//--------------------------------------------------------------------
	public int popByte ()
	{
		if (curDataPos < 6)
		{
			System.out.println ("Error: popByte before message start!");
			return 0;
		}
		int result = ByteConvert.toInt8 (data, curDataPos);
		curDataPos -= 1;
		return result;
	}

	//--------------------------------------------------------------------
	public boolean popBoolean ()
	{
		if (curDataPos < 6)
		{
			System.out.println ("Error: popBoolean before message start!");
			return false;
		}
		boolean result = ByteConvert.toBoolean (data, curDataPos);
		curDataPos -= 1;
		return result;
	}

	//--------------------------------------------------------------------
	public String popString ()
	{
		if (curDataPos - 1 < 6)
		{
			System.out.println ("Error: popString before message start!");
			return "";
		}
		String result = ByteConvert.toStringReverse (data, curDataPos);
		curDataPos -= (2 + result.length ());
		return result;
	}
	
	
	//--------------------------------------------------------------------
	public boolean writeToStream (OutputStream output)
	{
		data[0] = -1;
		ByteConvert.fromInt16 (msgLength, data, 1);
		ByteConvert.fromInt16 (msgType, data, 3);
		data[5] = (byte) playerNr;
		try
		{
			output.write (data, 0, msgLength);
		}
		catch (IOException e)
		{
			System.out.println ("Couldn't send msg: " + e);
			return false;
		}
		return true;
	}	
	
	//--------------------------------------------------------------------
	public String toString ()
	{
		return "msgType: " + getMessageTypeAsString (msgType) + "  msgLength: " + msgLength + "  playerNr: " + playerNr;
	}
	
	//--------------------------------------------------------------------
	protected void initMsg () 
	{
		if (data.length >= 6)
		{
			msgLength = ByteConvert.toInt16 (data, 1);
			msgType = ByteConvert.toInt16 (data, 3);
			playerNr = data[5];
			curDataPos = msgLength - 1;
		}
	}
	
	//--------------------------------------------------------------------
	public static String getMessageTypeAsString (int type)
	{
		switch (type)
		{
			case MU_MSG_CHAT:
				return "MU_MSG_CHAT";
			case MU_MSG_NEW_PLAYER:
				return "MU_MSG_NEW_PLAYER";
			case MU_MSG_REQ_IDENTIFIKATION:
				return "MU_MSG_REQ_IDENTIFIKATION";
			case MU_MSG_IDENTIFIKATION:
				return "MU_MSG_IDENTIFIKATION";
			case MU_MSG_DEL_PLAYER:
				return "MU_MSG_DEL_PLAYER";
			case MU_MSG_PLAYERLIST:
				return "MU_MSG_PLAYERLIST";
			case MU_MSG_OPTINS:
				return "MU_MSG_OPTINS";
			case MU_MSG_GO:
				return "MU_MSG_GO";
			case MU_MSG_CLAN:
				return "MU_MSG_CLAN";
			case MU_MSG_LANDING_VEHICLES:
				return "MU_MSG_LANDING_VEHICLES";
			case MU_MSG_UPGRADES:
				return "MU_MSG_UPGRADES";
			case MU_MSG_LANDING_COORDS:
				return "MU_MSG_LANDING_COORDS";
			case MU_MSG_RESELECT_LANDING:
				return "MU_MSG_RESELECT_LANDING";
			case MU_MSG_ALL_LANDED:
				return "MU_MSG_ALL_LANDED";
			case MU_MSG_START_MAP_DOWNLOAD:
				return "MU_MSG_START_MAP_DOWNLOAD";
			case MU_MSG_MAP_DOWNLOAD_DATA:
				return "MU_MSG_MAP_DOWNLOAD_DATA";
			case MU_MSG_CANCELED_MAP_DOWNLOAD:
				return "MU_MSG_CANCELED_MAP_DOWNLOAD";
			case MU_MSG_FINISHED_MAP_DOWNLOAD:
				return "MU_MSG_FINISHED_MAP_DOWNLOAD";
			case MU_MSG_REQUEST_MAP:
				return "MU_MSG_REQUEST_MAP";
			case GAME_EV_PLAYER_CLANS:
				return "GAME_EV_PLAYER_CLANS";
			case GAME_EV_ADD_BUILDING:
				return "GAME_EV_ADD_BUILDING";
			case GAME_EV_ADD_VEHICLE:
				return "GAME_EV_ADD_VEHICLE";
			case GAME_EV_DEL_BUILDING:
				return "GAME_EV_DEL_BUILDING";
			case GAME_EV_DEL_VEHICLE:
				return "GAME_EV_DEL_VEHICLE";
			case GAME_EV_ADD_ENEM_BUILDING:
				return "GAME_EV_ADD_ENEM_BUILDING";
			case GAME_EV_ADD_ENEM_VEHICLE:
				return "GAME_EV_ADD_ENEM_VEHICLE";
			case GAME_EV_CHAT_SERVER:
				return "GAME_EV_CHAT_SERVER";
			case GAME_EV_LOST_CONNECTION:
				return "GAME_EV_LOST_CONNECTION";
			case GAME_EV_CHAT_CLIENT:
				return "GAME_EV_CHAT_CLIENT";
			case GAME_EV_WANT_TO_END_TURN:
				return "GAME_EV_WANT_TO_END_TURN";
			case GAME_EV_MAKE_TURNEND:
				return "GAME_EV_MAKE_TURNEND";
			case GAME_EV_FINISHED_TURN:
				return "GAME_EV_FINISHED_TURN";
			case GAME_EV_UNIT_DATA:
				return "GAME_EV_UNIT_DATA";
			case GAME_EV_WANT_START_WORK:
				return "GAME_EV_WANT_START_WORK";
			case GAME_EV_WANT_STOP_WORK:
				return "GAME_EV_WANT_STOP_WORK";
			case GAME_EV_DO_START_WORK:
				return "GAME_EV_DO_START_WORK";
			case GAME_EV_DO_STOP_WORK:
				return "GAME_EV_DO_STOP_WORK";
			case GAME_EV_NEXT_MOVE:
				return "GAME_EV_NEXT_MOVE";
			case GAME_EV_MOVE_JOB_SERVER:
				return "GAME_EV_MOVE_JOB_SERVER";
			case GAME_EV_MOVE_JOB_CLIENT:
				return "GAME_EV_MOVE_JOB_CLIENT";
			case GAME_EV_WANT_STOP_MOVE:
				return "GAME_EV_WANT_STOP_MOVE";
			case GAME_EV_WANT_ATTACK:
				return "GAME_EV_WANT_ATTACK";
			case GAME_EV_ATTACKJOB_LOCK_TARGET:
				return "GAME_EV_ATTACKJOB_LOCK_TARGET";
			case GAME_EV_ATTACKJOB_FIRE:
				return "GAME_EV_ATTACKJOB_FIRE";
			case GAME_EV_ATTACKJOB_FINISHED:
				return "GAME_EV_ATTACKJOB_FINISHED";
			case GAME_EV_RESOURCES:
				return "GAME_EV_RESOURCES";
			case GAME_EV_MINELAYERSTATUS:
				return "GAME_EV_MINELAYERSTATUS";
			case GAME_EV_WANT_BUILD:
				return "GAME_EV_WANT_BUILD";
			case GAME_EV_BUILD_ANSWER:
				return "GAME_EV_BUILD_ANSWER";
			case GAME_EV_STOP_BUILD:
				return "GAME_EV_STOP_BUILD";
			case GAME_EV_END_BUILDING:
				return "GAME_EV_END_BUILDING";
			case GAME_EV_WANT_STOP_BUILDING:
				return "GAME_EV_WANT_STOP_BUILDING";
			case GAME_EV_SUBBASE_VALUES:
				return "GAME_EV_SUBBASE_VALUES";
			case GAME_EV_WANT_TRANSFER:
				return "GAME_EV_WANT_TRANSFER";
			case GAME_EV_WANT_BUILDLIST:
				return "GAME_EV_WANT_BUILDLIST";
			case GAME_EV_BUILDLIST:
				return "GAME_EV_BUILDLIST";
			case GAME_EV_WANT_EXIT_FIN_VEH:
				return "GAME_EV_WANT_EXIT_FIN_VEH";
			case GAME_EV_MINE_PRODUCE_VALUES:
				return "GAME_EV_MINE_PRODUCE_VALUES";
			case GAME_EV_CHANGE_RESOURCES:
				return "GAME_EV_CHANGE_RESOURCES";
			case GAME_EV_ATTACKJOB_IMPACT:
				return "GAME_EV_ATTACKJOB_IMPACT";
			case GAME_EV_TURN_REPORT:
				return "GAME_EV_TURN_REPORT";
			case GAME_EV_WANT_CHANGE_SENTRY:
				return "GAME_EV_WANT_CHANGE_SENTRY";
			case GAME_EV_ADD_RUBBLE:
				return "GAME_EV_ADD_RUBBLE";
			case GAME_EV_WANT_SUPPLY:
				return "GAME_EV_WANT_SUPPLY";
			case GAME_EV_SUPPLY:
				return "GAME_EV_SUPPLY";
			case GAME_EV_DETECTION_STATE:
				return "GAME_EV_DETECTION_STATE";
			case GAME_EV_CLEAR_ANSWER:
				return "GAME_EV_CLEAR_ANSWER";
			case GAME_EV_STOP_CLEARING:
				return "GAME_EV_STOP_CLEARING";
			case GAME_EV_NOFOG:
				return "GAME_EV_NOFOG";
			case GAME_EV_WANT_START_CLEAR:
				return "GAME_EV_WANT_START_CLEAR";
			case GAME_EV_WANT_STOP_CLEAR:
				return "GAME_EV_WANT_STOP_CLEAR";
			case GAME_EV_DEFEATED:
				return "GAME_EV_DEFEATED";
			case GAME_EV_ABORT_WAITING:
				return "GAME_EV_ABORT_WAITING";
			case GAME_EV_WAIT_RECON:
				return "GAME_EV_WAIT_RECON";
			case GAME_EV_ABORT_WAIT_RECON:
				return "GAME_EV_ABORT_WAIT_RECON";
			case GAME_EV_FREEZE:
				return "GAME_EV_FREEZE";
			case GAME_EV_UNFREEZE:
				return "GAME_EV_UNFREEZE";
			case GAME_EV_WAIT_FOR:
				return "GAME_EV_WAIT_FOR";
			case GAME_EV_DEL_PLAYER:
				return "GAME_EV_DEL_PLAYER";
			case GAME_EV_IDENTIFICATION:
				return "GAME_EV_IDENTIFICATION";
			case GAME_EV_RECON_SUCESS:
				return "GAME_EV_RECON_SUCESS";
			case GAME_EV_REQ_IDENT:
				return "GAME_EV_REQ_IDENT";
			case GAME_EV_RECON_IDENT:
				return "GAME_EV_RECON_IDENT";
			case GAME_EV_SPECIFIC_UNIT_DATA:
				return "GAME_EV_SPECIFIC_UNIT_DATA";
			case GAME_EV_TURN:
				return "GAME_EV_TURN";
			case GAME_EV_HUD_SETTINGS:
				return "GAME_EV_HUD_SETTINGS";
			case GAME_EV_WANT_LOAD:
				return "GAME_EV_WANT_LOAD";
			case GAME_EV_WANT_EXIT:
				return "GAME_EV_WANT_EXIT";
			case GAME_EV_STORE_UNIT:
				return "GAME_EV_STORE_UNIT";
			case GAME_EV_EXIT_UNIT:
				return "GAME_EV_EXIT_UNIT";
			case GAME_EV_UNIT_UPGRADE_VALUES:
				return "GAME_EV_UNIT_UPGRADE_VALUES";
			case GAME_EV_WANT_MARK_LOG:
				return "GAME_EV_WANT_MARK_LOG";
			case GAME_EV_MARK_LOG:
				return "GAME_EV_MARK_LOG";
			case GAME_EV_CREDITS_CHANGED:
				return "GAME_EV_CREDITS_CHANGED";
			case GAME_EV_UPGRADED_BUILDINGS:
				return "GAME_EV_UPGRADED_BUILDINGS";
			case GAME_EV_UPGRADED_VEHICLES:
				return "GAME_EV_UPGRADED_VEHICLES";
			case GAME_EV_RESEARCH_SETTINGS:
				return "GAME_EV_RESEARCH_SETTINGS";			
			case GAME_EV_RESEARCH_LEVEL:
				return "GAME_EV_RESEARCH_LEVEL";
			case GAME_EV_REFRESH_RESEARCH_COUNT:
				return "GAME_EV_REFRESH_RESEARCH_COUNT";
			case GAME_EV_WANT_VEHICLE_UPGRADE:
				return "GAME_EV_WANT_VEHICLE_UPGRADE";			
			case GAME_EV_WANT_BUY_UPGRADES:
				return "GAME_EV_WANT_BUY_UPGRADES";			
			case GAME_EV_WANT_BUILDING_UPGRADE:
				return "GAME_EV_WANT_BUILDING_UPGRADE";			
			case GAME_EV_WANT_RESEARCH_CHANGE:
				return "GAME_EV_WANT_RESEARCH_CHANGE";			
			case GAME_EV_AUTOMOVE_STATUS:
				return "GAME_EV_AUTOMOVE_STATUS";
			case GAME_EV_SET_AUTOMOVE:
				return "GAME_EV_SET_AUTOMOVE";
			case GAME_EV_WANT_COM_ACTION:
				return "GAME_EV_WANT_COM_ACTION";
			case GAME_EV_COMMANDO_ANSWER:
				return "GAME_EV_COMMANDO_ANSWER";
			case GAME_EV_REQ_SAVE_INFO:
				return "GAME_EV_REQ_SAVE_INFO";
			case GAME_EV_SAVED_REPORT:
				return "GAME_EV_SAVED_REPORT";
			case GAME_EV_SCORE:
				return "GAME_EV_SCORE";
			case GAME_EV_NUM_ECOS:
				return "GAME_EV_NUM_ECOS";
			case GAME_EV_UNIT_SCORE:
				return "GAME_EV_UNIT_SCORE";
			case GAME_EV_VICTORY_CONDITIONS:
				return "GAME_EV_VICTORY_CONDITIONS";
			case GAME_EV_SELFDESTROY:				
				return "GAME_EV_SELFDESTROY";
			case DEBUG_CHECK_VEHICLE_POSITIONS:
				return "DEBUG_CHECK_VEHICLE_POSITIONS";
			default:
				return String.valueOf (type);
		}
	}

}
