package msgs;

//--------------------------------------------------------------------
public class ByteConvert 
{

	//--------------------------------------------------------------------
	public static int toInt32 (byte[] inputBytes, int offset)
	{
		if (offset < 0 || offset+3 >= inputBytes.length)
		{
			System.out.println ("toInt32: Invalid offset: " + offset + "  inputBytes.length: " + inputBytes.length);
			return 0;
		}
		return ((int) inputBytes[offset] & 0xFF) 
			+ ((int) inputBytes[offset + 1] & 0xFF) * 256
			+ ((int) inputBytes[offset + 2] & 0xFF) * 65536
			+ ((int) inputBytes[offset + 3] & 0xFF) * 16777216;
	}

	//--------------------------------------------------------------------
	public static int toInt16 (byte[] inputBytes, int offset)
	{
		if (offset < 0 || offset + 1 >= inputBytes.length)
		{
			System.out.println ("toInt16: Invalid offset: " + offset + "  inputBytes.length: " + inputBytes.length);
			return 0;
		}
		return ((int) inputBytes[offset] & 0xFF) + ((int) inputBytes[offset + 1] & 0xFF) * 256;	
	}

	//--------------------------------------------------------------------
	public static int toInt8 (byte[] inputBytes, int offset)
	{
		if (offset < 0 || offset >= inputBytes.length)
		{
			System.out.println ("toInt8: Invalid offset: " + offset + "  inputBytes.length: " + inputBytes.length);
			return 0;
		}
		return ((int) inputBytes[offset] & 0xFF);	
	}

	//--------------------------------------------------------------------
	public static boolean toBoolean (byte[] inputBytes, int offset)
	{
		if (offset < 0 || offset >= inputBytes.length)
		{
			System.out.println ("toBoolean: Invalid offset: " + offset + "  inputBytes.length: " + inputBytes.length);
			return false;
		}
		return (inputBytes[offset] != 0);				
	}
	
	//--------------------------------------------------------------------
	public static String toString (byte[] inputBytes, int offset)
	{
		if (offset < 0 || offset + 1 >= inputBytes.length)
		{
			System.out.println ("toString: Invalid offset: " + offset + "  inputBytes.length: " + inputBytes.length);
			return "";
		}
		int curPos = offset + 1;
		while (curPos < inputBytes.length && inputBytes[curPos] != 0)
			curPos++;
		
		return new String (inputBytes, offset + 1, curPos - (offset + 1));
	}
	
	//--------------------------------------------------------------------
	public static String toStringReverse (byte[] inputBytes, int endOffset)
	{
		if (endOffset < 2 || endOffset >= inputBytes.length)
		{
			System.out.println ("toString: Invalid offset: " + endOffset + "  inputBytes.length: " + inputBytes.length);
			return "";
		}
		int curPos = endOffset - 1;
		while (curPos > 0 && inputBytes[curPos] != 0)
			curPos--;
		
		return new String (inputBytes, curPos + 1, endOffset - 1 - curPos);
	}
	
	//--------------------------------------------------------------------
	public static void fromInt32 (int value, byte[] outputBytes, int offset)
	{
		if (offset < 0 || offset + 3 >= outputBytes.length)
		{
			System.out.println ("fromInt32: Invalid offset: " + offset + "  outputBytes.length: " + outputBytes.length);
			return;
		}
		outputBytes[offset] = (byte) (value & 0xFF);
		outputBytes[offset + 1] = (byte) ((value & 0xFF00) >> 8);
		outputBytes[offset + 2] = (byte) ((value & 0xFF0000) >>16);
		outputBytes[offset + 3] = (byte) ((value & 0xFF000000) >> 24);
	}

	//--------------------------------------------------------------------
	public static void fromInt16 (int value, byte[] outputBytes, int offset)
	{
		if (offset < 0 || offset + 1 >= outputBytes.length)
		{
			System.out.println ("fromInt16: Invalid offset: " + offset + "  outputBytes.length: " + outputBytes.length);
			return;
		}
		outputBytes[offset] = (byte) (value & 0xFF);
		outputBytes[offset + 1] = (byte) ((value & 0xFF00) >> 8);
	}

	//--------------------------------------------------------------------
	public static void fromByte (int value, byte[] outputBytes, int offset)
	{
		if (offset < 0 || offset >= outputBytes.length)
		{
			System.out.println ("fromByte: Invalid offset: " + offset + "  outputBytes.length: " + outputBytes.length);
			return;
		}
		outputBytes[offset] = (byte) (value & 0xFF);
	}

	//--------------------------------------------------------------------
	public static void fromBoolean (boolean value, byte[] outputBytes, int offset)
	{
		if (offset < 0 || offset  >= outputBytes.length)
		{
			System.out.println ("fromBoolean: Invalid offset: " + offset + "  outputBytes.length: " + outputBytes.length);
			return;
		}
		outputBytes[offset] = (byte) (value ? 1 : 0);
	}
	
	//--------------------------------------------------------------------
	public static int fromString (String string, byte[] outputBytes, int offset)
	{
		if (string.length () + 2 >= 1024 - offset) // + 2 for \0 terminations
		{
			System.out.println ("fromString: Size of string exceeds MAX_MESSAGE_LENGTH: " + string.length ());
			return offset;
		}
		byte[] stringBytes = string.getBytes ();
		outputBytes[offset] = 0;
		System.arraycopy (stringBytes, 0, outputBytes, offset + 1, string.length ());
		outputBytes[offset + 1 + string.length ()] = 0;
		return offset + 1 + string.length () + 1;
	}
	
}
