package /*max3.common.*/msgs;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;


//--------------------------------------------------------------------
public class MessageFactory {

	public static final int MAX_MESSAGE_LENGTH = 1024;
	public static final int MIN_MESSAGE_LENGTH = 6;
	public static final int MSG_TYPE_OFFSET = 3;
	public static final int MSG_LENGTH_OFFSET = 1;
	
	//--------------------------------------------------------------------
	public static ArrayList<Message> createMsgsFromStream (InputStream input)
	throws IOException
	{
		ArrayList<Message> result = null;
		if (input.available () > 0)
		{
			int bytesToRead = input.available ();
			byte[] totalInputBytes = new byte[bytesToRead];
			int bytesRead = input.read (totalInputBytes);
			
			// totalInputBytes now holds the data of one or mores messages. Now create the message(s).
			int curPos = 0;
			while (curPos < bytesRead && bytesRead - curPos >= MIN_MESSAGE_LENGTH)
			{
				int msgLength = ByteConvert.toInt16 (totalInputBytes, curPos + MSG_LENGTH_OFFSET);
				if (msgLength > MAX_MESSAGE_LENGTH)
					throw new IOException ("MsgLength exceeds MAX_MESSAGE_LENGTH: " + msgLength);
				if (msgLength < MIN_MESSAGE_LENGTH)
					throw new IOException ("MsgLength too small: " + msgLength);
				if (msgLength > bytesToRead - curPos)
					throw new IOException ("MsgLength (" + msgLength + ") is greater than BytesToRead-curPos (" + (bytesToRead - curPos) + "): " + toHexDump (totalInputBytes, curPos));

				byte[] msgBytes = new byte[msgLength];
				System.arraycopy (totalInputBytes, curPos, msgBytes, 0, msgLength);
				curPos += msgLength;
				
				if (result == null)
					result = new ArrayList<Message>();				

				Message newMsg;
				int msgType = ByteConvert.toInt16 (msgBytes, MSG_TYPE_OFFSET);
				switch (msgType)
				{
					case Message.MU_MSG_CHAT:
					case Message.MU_MSG_NEW_PLAYER:
					case Message.MU_MSG_REQ_IDENTIFIKATION: 
					case Message.MU_MSG_PLAYERLIST:
					case Message.MU_MSG_OPTINS:
					case Message.MU_MSG_GO:
					case Message.MU_MSG_LANDING_VEHICLES:
					case Message.MU_MSG_UPGRADES:
					case Message.MU_MSG_LANDING_COORDS:
					case Message.MU_MSG_RESELECT_LANDING:
					case Message.MU_MSG_ALL_LANDED:
					case Message.GAME_EV_ADD_VEHICLE:
					case Message.GAME_EV_ADD_ENEM_VEHICLE:
					case Message.GAME_EV_UNIT_DATA:
					case Message.GAME_EV_NEXT_MOVE:
					case Message.GAME_EV_FINISHED_TURN:
					case Message.GAME_EV_ATTACKJOB_LOCK_TARGET:
					case Message.GAME_EV_ATTACKJOB_FIRE:
					case Message.GAME_EV_ATTACKJOB_IMPACT:
					case Message.GAME_EV_ATTACKJOB_FINISHED:
					case Message.GAME_EV_RESOURCES:
					case Message.GAME_EV_REQUEST_RESYNC:
						newMsg = new BasicMsg (msgBytes); 
						break;
					default:
						System.out.print ("Warning: Unknown message arrived! MsgType: "); 
						System.out.println (BasicMsg.getMessageTypeAsString (msgType));
						newMsg = new BasicMsg (msgBytes); 
						break;
				}
				System.out.println ("Received: " + newMsg);
				result.add (newMsg);
			}
		}
		return result;
	}
	
	//--------------------------------------------------------------------
	private static String toHexDump (byte[] data, int offset)
	{
		String result = "";
		for (int i = offset; i < data.length; i++)
		{
			byte higher = (byte) (data[i] / 16);
			byte lower = (byte) (data[i] % 16);
			switch (higher)
			{
			case 0: result += "0"; break;
			case 1: result += "1"; break;
			case 2: result += "2"; break;
			case 3: result += "3"; break;
			case 4: result += "4"; break;
			case 5: result += "5"; break;
			case 6: result += "6"; break;
			case 7: result += "7"; break;
			case 8: result += "8"; break;
			case 9: result += "9"; break;
			case 10: result += "A"; break;
			case 11: result += "B"; break;
			case 12: result += "C"; break;
			case 13: result += "D"; break;
			case 14: result += "E"; break;
			case 15: result += "F"; break;
			}
			switch (lower)
			{
			case 0: result += "0"; break;
			case 1: result += "1"; break;
			case 2: result += "2"; break;
			case 3: result += "3"; break;
			case 4: result += "4"; break;
			case 5: result += "5"; break;
			case 6: result += "6"; break;
			case 7: result += "7"; break;
			case 8: result += "8"; break;
			case 9: result += "9"; break;
			case 10: result += "A"; break;
			case 11: result += "B"; break;
			case 12: result += "C"; break;
			case 13: result += "D"; break;
			case 14: result += "E"; break;
			case 15: result += "F"; break;
			}
		}
		return new String (result);
	}
	
}
